package eu.jonahbauer.wizard.core.machine.states;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.machine.GameData;
import eu.jonahbauer.wizard.core.messages.player.PlayerMessage;
import lombok.Getter;
import lombok.experimental.Delegate;
import org.jetbrains.annotations.Unmodifiable;

import java.util.UUID;

@Unmodifiable
public abstract class GameState implements State {
    @Getter
    private final Game game;
    @Getter
    @Delegate(types = GameData.Getters.class)
    private final GameData data;

    public GameState(Game game, GameData data) {
        this.game = game;
        this.data = data.requirePlayers().onlyRequired();
    }

    protected final void transition(GameState state) {
        getGame().transition(this, state);
    }

    protected final void timeout() {
        getGame().timeout(this, getTimeout(false));
    }

    protected final long getTimeout(boolean absolute) {
        return (absolute ? System.currentTimeMillis() : 0) + getGame().getConfig().timeout();
    }

    public void onMessage(UUID player, PlayerMessage message) {
        throw new IllegalStateException("You cannot do that right now.");
    }

    @Override
    public String toString() {
        return getClass().getSimpleName();
    }
}
