package eu.jonahbauer.wizard.core.machine.states.round;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.machine.GameData;
import eu.jonahbauer.wizard.core.machine.states.game.Finishing;
import eu.jonahbauer.wizard.core.messages.observer.ScoreMessage;
import org.jetbrains.annotations.VisibleForTesting;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public final class FinishingRound extends RoundState {
    private transient Map<UUID, Integer> points;

    public FinishingRound(Game game, GameData data) {
        super(game, data.requireAllPredictions().requireTricks().requireScore());

        int tricks = data.getTricks().values().stream().mapToInt(i -> i).sum();
        if (tricks != data.getRound() + 1) {
            throw new AssertionError("Unexpected number of tricks in round " + data.getRound() + ": " + tricks + ".");
        }
    }

    @Override
    public void onEnter() {
        points = getPoints();

        var score = new HashMap<>(getScore());
        points.forEach((uuid, p) -> score.compute(uuid, (u, total) -> total == null ? p : total + p));

        GameData data = getData().withScore(score);

        if (60 / getPlayerCount() == getRound() + 1) {
            transition(new Finishing(getGame(), data));
        } else {
            transition(new StartingRound(getGame(), data.withNextRound()));
        }
    }

    @Override
    public void onExit() {
        if (points != null) {
            getGame().notify(new ScoreMessage(points));
        }
    }

    @VisibleForTesting
    Map<UUID, Integer> getPoints() {
        var points = new HashMap<UUID, Integer>();
        for (UUID player : getPlayers()) {
            int prediction = getPredictions().get(player);
            int tricks = getTricks().getOrDefault(player, 0);

            if (tricks == prediction) {
                points.put(player, 20 + 10 * tricks);
            } else {
                points.put(player, -10 * Math.abs(tricks - prediction));
            }
        }
        return points;
    }
}
